@@ -1,6 +1,6 @@
 package CPAN::Index::API::Role::Reader;
 {
-  $CPAN::Index::API::Role::Reader::VERSION = '0.002';
+  $CPAN::Index::API::Role::Reader::VERSION = '0.003';
 }
 
 # ABSTRACT: Role for reading index files
@@ -12,12 +12,13 @@ use File::Temp     qw(tempfile);
 use Scalar::Util   qw(blessed);
 use Path::Class    qw(file);
 use Carp           qw(croak);
+use LWP::Simple;
 use Compress::Zlib qw(gzopen Z_STREAM_END), '$gzerrno';
 use Moose::Role;
 use namespace::clean -except => 'meta';
 
 requires 'parse';
-requires 'default_locations';
+requires 'default_location';
 
 sub read_from_string
 {
@@ -55,7 +56,7 @@ sub read_from_tarball
     $content .= $buffer while $gz->gzread($buffer) > 0 ;
 
     croak "Error reading from $tarball: $gzerrno" . ($gzerrno+0) . "\n"
-        if $gzerrno != Z_STREAM_END ;
+        if $gzerrno != Z_STREAM_END;
 
     $gz->gzclose and croak "Error closing $tarball";
 
@@ -68,14 +69,8 @@ sub read_from_repo_path
 
     $args{repo_path} = $repo_path;
 
-    my @default_locations = $self->default_locations;
-
-    my $path_to_file = file(
-        $repo_path, @{ $default_locations[0] }
-    )->stringify;
-
     return $self->read_from_tarball(
-        $path_to_file, %args
+        file( $repo_path, $self->default_location )->stringify, %args
     );
 }
 
@@ -86,9 +81,12 @@ sub read_from_repo_uri
     $args{repo_uri} = $repo_uri;
 
     my $uri = URI->new( $repo_uri );
-    my @default_locations = $self->default_locations;
 
-    $uri->path_segments( $uri->path_segments, @{ $default_locations[0] } );
+    $uri->path_segments(
+        $uri->path_segments,
+        file($self->default_location)->dir->dir_list,
+        file($self->default_location)->basename,
+    );
 
     my $uri_as_string = $uri->as_string;
 
@@ -104,6 +102,7 @@ sub read_from_repo_uri
 
 1;
 
+
 __END__
 =pod
 
@@ -113,7 +112,49 @@ CPAN::Index::API::Role::Reader - Role for reading index files
 
 =head1 VERSION
 
-version 0.002
+version 0.003
+
+=head1 DESCRIPTION
+
+This role provides a collection of utility constructors for CPAN index file
+objects.
+
+=head1 REQUIRES
+
+=head2 default_location
+
+Class method that returns a string specifying the path to the default location
+of this file relative to the repository root.
+
+=head2 parse
+
+This class method (generally invoked as part of the construction phase)
+should accept a string containing an index file, and return a list of
+key/value pairs suitable for passing to the constructor of the consuming class.
+
+=head1 PROVIDES
+
+=head2 read_from_string
+
+Construct a new index file object by reading the file contents from a string.
+
+=head2 read_from_file
+
+Construct a new index file object by reading the file contents from a filename.
+
+=head2 read_from_tarball
+
+Construct a new index file object by reading the file contents from a tarball.
+
+=head2 read_from_repo_path
+
+Construct a new index file object by locating and parsing a file in a local
+repository.
+
+=head2 read_from_repo_uri
+
+Construct a new index file object by locating and parsing a file in a remote
+repository.
 
 =head1 AUTHOR
 
